@@ -7,7 +7,7 @@
 //!
 //! ```toml
 //! [dependencies]
-//! static_assertions = "0.1"
+//! static_assertions = "0.2.0"
 //! ```
 //!
 //! and this to your crate root:
@@ -29,12 +29,18 @@
 //! ```
 //! # #[macro_use]
 //! # extern crate static_assertions;
-//! # fn main() {
-//! assert_eq_size!([u8; 4], (u16, u16), u32);
+//! // Can be declared outside of a function if labeled
+//! assert_eq_size!(bytes; (u8, u8), u16);
 //!
-//! // Produces a compilation failure:
-//! // assert_eq_size!(u32, u8);
-//! # }
+//! // Fails to compile (same label):
+//! // assert_eq_size!(bytes; u8, u8);
+//!
+//! fn main() {
+//!     assert_eq_size!([u8; 4], (u16, u16), u32);
+//!
+//!     // Produces a compilation failure:
+//!     // assert_eq_size!(u32, u8);
+//! }
 //! ```
 //!
 //! Similar to [`assert_eq_size`], there is [`assert_eq_size_val`]. Instead of
@@ -69,12 +75,18 @@
 //! # }
 //! ```
 //!
+//! Rather than dereference a pointer to achieve the same effect as
+//! [`assert_eq_size_val`], there is also the option of [`assert_eq_size_ptr`].
+//!
 //! # Assert Constant Expression
 //!
 //! Constant expressions can be ensured to have certain properties via
 //! [`const_assert`]. If the expression evaluates to `false`, the file will fail
 //! to compile. This is synonymous to [`static_assert` in C++][static_assert].
 //!
+//! As a [limitation](#limitations), a unique label is required if the macro is
+//! used outside of a function.
+//!
 //! ```
 //! # #[macro_use]
 //! # extern crate static_assertions;
@@ -90,19 +102,30 @@
 //! ```
 //! # #[macro_use]
 //! # extern crate static_assertions;
-//! # fn main() {
-//! const NUM: usize = 32;
-//! const_assert_eq!(NUM + NUM, 64);
-//!
 //! const TWO: usize = 2;
-//! const_assert_eq!(TWO * TWO, TWO + TWO, 4);
-//! # }
+//! const_assert_eq!(two; TWO * TWO, TWO + TWO, 4);
+//!
+//! // Produces a compilation failure:
+//! // const_assert_eq!(two; TWO, TWO);
+//!
+//! fn main() {
+//!     const NUM: usize = 32;
+//!     const_assert_eq!(NUM + NUM, 64);
+//! }
 //! ```
 //!
-//! **Limitation:** Due to implementation details, [`assert_eq_size`],
-//! [`const_assert`], and [`const_assert_eq`] can only be used from within the
-//! context of a function.
+//! # Limitations
 //!
+//! Due to implementation details, [`assert_eq_size`], [`const_assert`], and
+//! [`const_assert_eq`] can only be used normally from within the context of a
+//! function.
+//!
+//! To use these macros in other contexts, a unique label must be provided.
+//!
+//! If you want to read up about this and provide feedback, see
+//! [the related issue on GitHub][issue1].
+//!
+//! [issue1]: https://github.com/nvzqz/static-assertions-rs/issues/1
 //! [crate]: https://crates.io/crates/static_assertions
 //! [static_assert]: http://en.cppreference.com/w/cpp/language/static_assert
 //! [`Clone`]: https://doc.rust-lang.org/std/clone/trait.Clone.html
@@ -110,6 +133,7 @@
 //! [`u64`]: https://doc.rust-lang.org/std/primitive.u64.html
 //! [`u32`]: https://doc.rust-lang.org/std/primitive.u32.html
 //! [`assert_eq_size_val`]: macro.assert_eq_size_val.html
+//! [`assert_eq_size_ptr`]: macro.assert_eq_size_ptr.html
 //! [`assert_eq_size`]: macro.assert_eq_size.html
 //! [`const_assert`]: macro.const_assert.html
 //! [`const_assert_eq`]: macro.const_assert_eq.html
@@ -121,9 +145,6 @@ pub extern crate core as _core;
 
 /// Asserts at compile-time that the types have equal sizes.
 ///
-/// This especially is useful for when coercing pointers between different types
-/// and ensuring the underlying values are the same size.
-///
 /// # Example
 ///
 /// ```
@@ -131,15 +152,20 @@ pub extern crate core as _core;
 /// # extern crate static_assertions;
 /// struct Byte(u8);
 ///
-/// # fn main() {
-/// assert_eq_size!(Byte, u8);
+/// assert_eq_size!(pair; (u16, u16), [u16; 2], [u8; 4]);
 ///
-/// // Supports unlimited arguments:
-/// assert_eq_size!([Byte; 4], [u16; 2], u32);
+/// // Fails to compile (same label):
+/// // assert_eq_size!(pair; u8, u8);
 ///
-/// // Fails to compile:
-/// // assert_eq_size!(Byte, u16);
-/// # }
+/// fn main() {
+///     assert_eq_size!(Byte, u8);
+///
+///     // Supports unlimited arguments:
+///     assert_eq_size!([Byte; 4], [u16; 2], u32);
+///
+///     // Produces a compilation failure:
+///     // assert_eq_size!(Byte, u16);
+/// }
 /// ```
 #[macro_export]
 macro_rules! assert_eq_size {
@@ -149,6 +175,38 @@ macro_rules! assert_eq_size {
             use $crate::_core::mem::{forget, transmute, uninitialized};
             $(forget::<$xs>(transmute(uninitialized::<$x>()));)+
         }
+    };
+    ($label:ident; $x:ty, $($xs:ty),+) => {
+        #[allow(dead_code, non_snake_case)]
+        fn $label() { assert_eq_size!($x, $($xs),+) }
+    };
+}
+
+/// Asserts at compile-time that the values pointed to have equal sizes.
+///
+/// This especially is useful for when coercing pointers between different types
+/// and ensuring the underlying values are the same size.
+///
+/// # Example
+///
+/// ```
+/// # #[macro_use]
+/// # extern crate static_assertions;
+/// fn operation(x: &(u32, u32), y: &[u16; 4]) {
+///     assert_eq_size_ptr!(x, y);
+/// }
+/// # fn main() {}
+/// ```
+#[macro_export]
+macro_rules! assert_eq_size_ptr {
+    ($x:expr, $($xs:expr),+) => {
+        #[allow(unused_unsafe)]
+        unsafe {
+            use $crate::_core::{mem, ptr};
+            let mut copy = ptr::read($x);
+            $(ptr::write(&mut copy, mem::transmute(ptr::read($xs)));)+
+            mem::forget(copy);
+        }
     }
 }
 
@@ -175,13 +233,7 @@ macro_rules! assert_eq_size {
 #[macro_export]
 macro_rules! assert_eq_size_val {
     ($x:expr, $($xs:expr),+) => {
-        #[allow(unused_unsafe)]
-        unsafe {
-            use $crate::_core::{mem, ptr};
-            let mut copy = ptr::read(&$x);
-            $(ptr::write(&mut copy, mem::transmute(ptr::read(&$xs)));)+
-            mem::forget(copy);
-        }
+        assert_eq_size_ptr!(&$x, $(&$xs),+);
     }
 }
 
@@ -198,21 +250,18 @@ macro_rules! assert_eq_size_val {
 /// const FIVE: usize = 5;
 /// const_assert!(FIVE - FIVE == 0);
 ///
-/// // Fails to compile:
+/// // Produces a compilation failure:
 /// // const_assert!(1 >= 2);
 /// # }
 /// ```
 #[macro_export]
 macro_rules! const_assert {
-    ($cond:expr) => {
-        // Causes overflow if condition is false
-        let _ = [(); 0 - (!($cond) as usize)];
-    };
     ($($xs:expr),+) => {
-        const_assert!($($xs)&&+);
+        let _ = [(); 0 - (!($($xs)&&+) as usize)];
     };
-    ($($xs:expr);+ $(;)*) => {
-        const_assert!($($xs),+);
+    ($label:ident; $($xs:expr),+) => {
+        #[allow(dead_code, non_snake_case)]
+        fn $label() { const_assert!($($xs),+); }
     };
 }
 
@@ -221,5 +270,8 @@ macro_rules! const_assert {
 macro_rules! const_assert_eq {
     ($x:expr, $($xs:expr),+) => {
         const_assert!($($x == $xs),+);
-    }
+    };
+    ($label:ident; $x:expr, $($xs:expr),+) => {
+        const_assert!($label; $($x == $xs),+);
+    };
 }
